// https://github.com/espressif/arduino-esp32/issues/878

#include <esp_now.h>
#include <WiFi.h>
#include <PubSubClient.h>

#define CHANNEL 1

/** MQTT broker URL */
static const char * mqttBroker = "rubi.xoic.coop";
/** MQTT user name */
static const char * mqttUser = "rubi";
/** MQTT password */
static const char * mqttPwd = "iotRubi";
/** WiFi client class to receive messages from mqtt broker */
WiFiClient espClient;
PubSubClient client(espClient);

#define MAC_SIZE 15
char sMac[MAC_SIZE];

#define I2C_SDA 21
#define I2C_SCL 22

#include "SSD1306.h" // alias for `#include "SSD1306Wire.h"`
SSD1306 display(0x3c, I2C_SDA, I2C_SCL);

// Structure example to receive data
// Must match the sender structure
typedef struct struct_message {
  int id;
  int x;
  int y;
} struct_message;

// Create a struct_message called myData
struct_message myData;

// Create a structure to hold the readings from each board
struct_message board1;
struct_message board2;
struct_message board3;
struct_message board4;
struct_message board5;

// Create an array with all the structures
struct_message boardsStruct[5] = {board1, board2, board3, board4, board5};

// callback function that will be executed when data is received
void OnDataRecv(const uint8_t * mac_addr, const uint8_t *incomingData, int len) {
  char macStr[18], missatge[99], tema[55];
  Serial.print("Packet received from: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.println(macStr);
  memcpy(&myData, incomingData, sizeof(myData));
  Serial.printf("Board ID %u: %u bytes\n", myData.id, len);
  // Update the structures with the new incoming data
  boardsStruct[myData.id - 1].x = myData.x;
  boardsStruct[myData.id - 1].y = myData.y;
  Serial.printf("x value: %d \n", boardsStruct[myData.id - 1].x);
  Serial.printf("y value: %d \n", boardsStruct[myData.id - 1].y);
  Serial.println();
  sprintf(missatge, "(%d,%d)", myData.x, myData.y);
  if (myData.id < 10) {
    sprintf(tema, "/%s/placa0%d", sMac, myData.id);
  } else {
    sprintf(tema, "/%s/placa%d", sMac, myData.id);
  }
  Serial.print("tema: ");
  Serial.print(tema);
  Serial.print(" | missatge: ");
  Serial.println(missatge);
  client.publish(tema, missatge);
}

void vSetupScreen() {
  display.init();
  display.flipScreenVertically();
}

// config AP SSID
void configDeviceAP() {
  String Prefix = "ESP32_";
  String Mac = WiFi.macAddress();
  String SSID = Prefix + Mac;
  String Password = "123456789";
  bool result = WiFi.softAP(SSID.c_str(), Password.c_str(), CHANNEL, 0);
  if (!result) {
    Serial.println("AP Config failed.");
  } else {
    Serial.println("AP Config Success. Broadcasting with AP: " + String(SSID));
  }
}

void setupWifi() {
  WiFi.mode(WIFI_AP_STA);
  WiFi.begin("CASTELLA40_2G", "1A2B3C4D5E");
  // configure device AP mode
  configDeviceAP();
  // This is the mac address of the Slave in AP Mode
  Serial.print("AP MAC: "); Serial.println(WiFi.softAPmacAddress());
}

// Init ESP Now with fallback
void InitESPNow() {
  if (esp_now_init() == ESP_OK) {
    Serial.println("ESPNow Init Success");
  }
  else {
    Serial.println("ESPNow Init Failed");
    ESP.restart();
  }
}

void setupMqtt() {
  // Start connection to MQTT broker
  // Connect to MQTT broker
  client.setServer(mqttBroker, 1883);
  //client.setCallback(receivedCallback);

  Serial.println("Connecting to MQTT broker");

  int connectTimeout = 0;

  while (!client.connected()) {
    delay(100);
    String clientId = "IoT-02_" + String(sMac); 
    if (client.connect(clientId.c_str(), mqttUser, mqttPwd)) {
      Serial.println("MQTT connected!");
      //client.subscribe(TOPIC_REQUEST_MAC); // <-------   Subscription to MQTT(S) topic
      //client.subscribe(String("/" + String(sMac) + TOPIC_LED_W).c_str());
    } else {
      connectTimeout++;
      if (connectTimeout > 10) { // Wait for 1 seconds to connect
        Serial.println("Can't connect to MQTT broker");
        ESP.restart();
      }
    }
  }
  Serial.println("Connected to MQTT");
}

String szGetMac() {
  byte mac[6];
  String szMAC = "";
  char szMac[3];

  WiFi.macAddress(mac);
  //for (int i = 5; i >= 0; i--)
  for (int i = 0; i < 6; i++)
  {
    if (mac[i] > 0x0F)
      sprintf(szMac, "%2X", mac[i]);
    else
      sprintf(szMac, "0%X", mac[i]);
    szMAC += szMac;
  }

  return szMAC;
}

void setup() {
  //Initialize Serial Monitor
  Serial.begin(115200);
  vSetupScreen();

  setupWifi();
  //Set device as a Wi-Fi Station
  //WiFi.mode(WIFI_STA);
  szGetMac().toCharArray(sMac, MAC_SIZE);
  setupMqtt();

  //Init ESP-NOW
  InitESPNow();
  // Once ESPNow is successfully Init, we will register for recv CB to
  // get recv packer info
  esp_now_register_recv_cb(OnDataRecv);
}

void loop() {
  String sText;
  // Acess the variables for each board
  /*int board1X = boardsStruct[0].x;
    int board1Y = boardsStruct[0].y;
    int board2X = boardsStruct[1].x;
    int board2Y = boardsStruct[1].y;
    int board3X = boardsStruct[2].x;
    int board3Y = boardsStruct[2].y;*/

  display.clear();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_10);
  sText = "Placa 1: ( " + String(boardsStruct[0].x) + " , " + String(boardsStruct[0].y) + " )";
  display.drawString(10, 10, sText);
  sText = "Placa 5: ( " + String(boardsStruct[4].x) + " , " + String(boardsStruct[4].y) + " )";
  display.drawString(10, 50, sText);
  display.display();

  delay(500);
}
